/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.editor.spi;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.io.Writer;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Locale;
import org.jdtaus.banking.dtaus.Checksum;
import org.jdtaus.banking.dtaus.Header;
import org.jdtaus.banking.dtaus.PhysicalFile;
import org.jdtaus.banking.dtaus.Transaction;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Dependency;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;
import org.jdtaus.core.container.PropertyException;
import org.jdtaus.core.monitor.spi.Task;
import org.jdtaus.core.monitor.spi.TaskMonitor;
import org.jdtaus.core.text.Message;
import org.jdtaus.editor.PhysicalFileFormatter;
import org.jdtaus.editor.spi.messages.CreatingTextFileMessage;

/**
 * {@code PhysicalFileFormatter} reference implementation producing a
 * human-readable text file reproducing the contents of a {@code PhysicalFile}.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: PhysicalFileTextFormatter.java 3512 2007-05-16 05:57:21Z schulte2005 $
 */
public final class PhysicalFileTextFormatter
    implements PhysicalFileFormatter, ContainerInitializer
{
    //--Implementation----------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(PhysicalFileTextFormatter.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /**
     * Protected <code>PhysicalFileTextFormatter</code> implementation constructor.
     *
     * @param meta Implementation meta-data.
     *
     * @throws NullPointerException if <code>meta</code> is <code>null</code>.
     */
    protected PhysicalFileTextFormatter(final Implementation meta)
    {
        super();
        if(meta == null)
        {
            throw new NullPointerException("meta");
        }
        this.initializeProperties(meta.getProperties());
    }
    /**
     * Protected <code>PhysicalFileTextFormatter</code> dependency constructor.
     *
     * @param meta dependency meta-data.
     *
     * @throws NullPointerException if <code>meta</code> is <code>null</code>.
     */
    protected PhysicalFileTextFormatter(final Dependency meta)
    {
        super();
        if(meta == null)
        {
            throw new NullPointerException("meta");
        }
        this.initializeProperties(meta.getProperties());
    }

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    protected void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

        p = meta.getProperty("transactionSeparator");
        this._transactionSeparator = (java.lang.String) p.getValue();


        p = meta.getProperty("fileSeparator");
        this._fileSeparator = (java.lang.String) p.getValue();


        p = meta.getProperty("fileExtension");
        this._fileExtension = (java.lang.String) p.getValue();

    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /** Initializes the instance. */
    public void initialize()
    {
        this.assertValidProperties();
    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Configured <code>TaskMonitor</code> implementation. */
    private transient TaskMonitor _dependency0;

    /**
     * Gets the configured <code>TaskMonitor</code> implementation.
     *
     * @return the configured <code>TaskMonitor</code> implementation.
     */
    private TaskMonitor getTaskMonitor()
    {
        TaskMonitor ret = null;
        if(this._dependency0 != null)
        {
            ret = this._dependency0;
        }
        else
        {
            ret = (TaskMonitor) ContainerFactory.getContainer().
                getDependency(PhysicalFileTextFormatter.class,
                "TaskMonitor");

            if(ModelFactory.getModel().getModules().
                getImplementation(PhysicalFileTextFormatter.class.getName()).
                getDependencies().getDependency("TaskMonitor").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /**
     * Property {@code transactionSeparator}.
     * @serial
     */
    private java.lang.String _transactionSeparator;

    /**
     * Gets the value of property <code>transactionSeparator</code>.
     *
     * @return the value of property <code>transactionSeparator</code>.
     */
    protected java.lang.String getTransactionSeparator()
    {
        return this._transactionSeparator;
    }

    /**
     * Property {@code fileSeparator}.
     * @serial
     */
    private java.lang.String _fileSeparator;

    /**
     * Gets the value of property <code>fileSeparator</code>.
     *
     * @return the value of property <code>fileSeparator</code>.
     */
    protected java.lang.String getFileSeparator()
    {
        return this._fileSeparator;
    }

    /**
     * Property {@code fileExtension}.
     * @serial
     */
    private java.lang.String _fileExtension;

    /**
     * Gets the value of property <code>fileExtension</code>.
     *
     * @return the value of property <code>fileExtension</code>.
     */
    protected java.lang.String getFileExtension()
    {
        return this._fileExtension;
    }


    //--------------------------------------------------------------Properties--
    //--PhysicalFileFormatter---------------------------------------------------

    public boolean accept(final File file)
    {
        if(file == null)
        {
            throw new NullPointerException("file");
        }

        return file.isFile() &&
            (file.getAbsolutePath().toLowerCase(Locale.getDefault()).
            endsWith('.' + this.getFileExtension()));

    }

    public String getDescription(final Locale locale)
    {
        return PhysicalFileTextFormatterBundle.getDescriptionText(locale);
    }

    public void format(final PhysicalFile pFile, final File toFile)
    throws IOException
    {
        if(pFile == null)
        {
            throw new NullPointerException("pFile");
        }
        if(toFile == null)
        {
            throw new NullPointerException("toFile");
        }

        final Task task = new Task();
        final BufferedWriter writer = new BufferedWriter(
            new FileWriter(toFile));

        long numTransactions = 0L;
        for(int i = 0; i < pFile.count(); i++)
        {
            numTransactions += pFile.get(i).getChecksum().getTransactionCount();
        }

        final int maxProgress = numTransactions > Integer.MAX_VALUE ?
            Integer.MAX_VALUE : (int) numTransactions;

        long progress = 0L;
        task.setIndeterminate(false);
        task.setCancelable(true);
        task.setDescription(new CreatingTextFileMessage());
        task.setMinimum(0);
        task.setMaximum(maxProgress);
        task.setProgress((int) progress);

        try
        {
            this.getTaskMonitor().monitor(task);
            for(int file = 0; file < pFile.count() &&
                !task.isCancelled(); file++)
            {
                writer.append(this.getFileSeparator()).append('\n');
                this.appendHeader(pFile.get(file).getHeader(), writer);
                this.appendChecksum(pFile.get(file).getChecksum(), writer);
                writer.append(this.getFileSeparator()).append('\n');

                for(int transaction = 0; transaction < pFile.get(file).
                    getChecksum().getTransactionCount() &&
                    !task.isCancelled(); transaction++)
                {
                    this.appendTransaction(pFile.get(file).
                        getTransaction(transaction), writer);

                    writer.append(this.getTransactionSeparator()).append('\n');
                    if(++progress < maxProgress)
                    {
                        task.setProgress((int) progress);
                    }
                }
            }
        }
        finally
        {
            if(task.isCancelled())
            {
                new RandomAccessFile(toFile, "rw").setLength(0L);
            }

            this.getTaskMonitor().finish(task);
        }

        writer.flush();
    }

    //---------------------------------------------------PhysicalFileFormatter--
    //--PhysicalFileTextFormatter-----------------------------------------------

    /**
     * Checks configured properties.
     *
     * @throws PropertyException for illegal property values.
     */
    private void assertValidProperties()
    {
        if(this.getFileExtension() == null ||
            this.getFileExtension().length() <= 0)
        {
            throw new PropertyException("fileExtension",
                this.getFileExtension());

        }
    }

    private void appendHeader(final Header h, final Writer writer)
    throws IOException
    {
        final NumberFormat ref10Fmt = new DecimalFormat("0000000000");
        final NumberFormat blzFmt = new DecimalFormat("00000000");
        final NumberFormat acctFmt = new DecimalFormat("0000000000");
        final DateFormat dateFmt = DateFormat.getDateInstance();

        writer.append(h.getReference() == null ?
            "          " : ref10Fmt.format(h.getReference())).append(' ');

        writer.append(h.getCustomer()).append(' ');
        writer.append(blzFmt.format(h.getBank())).append(' ');
        writer.append(acctFmt.format(h.getAccount())).append("\n");

        writer.append(h.getType().getShortDescription(Locale.getDefault())).
            append(' ');

        writer.append(h.getCurrency().getCurrencyCode()).append(" (").
            append(h.getCurrency().getSymbol(Locale.getDefault())).
            append(") ");

        writer.append(dateFmt.format(h.getCreateDate()));

        if(h.getExecutionDate() != null)
        {
            writer.append(' ').append(dateFmt.format(h.getExecutionDate()));
        }

        if(h.getBankData() != null)
        {
            writer.append(' ').append(blzFmt.format(h.getBankData()));
        }

        writer.append('\n');
    }

    private void appendChecksum(final Checksum c, final Writer writer)
    throws IOException
    {
        final NumberFormat countFmt = new DecimalFormat("0000000");
        final NumberFormat sumAcctFmt = new DecimalFormat("00000000000000000");
        final NumberFormat sumBankFmt = new DecimalFormat("00000000000000000");
        final NumberFormat sumAmountFmt = new DecimalFormat("0000000000000");

        writer.append(countFmt.format(c.getTransactionCount())).append(' ').
            append(sumAcctFmt.format(c.getSumAmount())).append(' ').
            append(sumBankFmt.format(c.getSumTargetBank())).append(' ').
            append(sumAmountFmt.format(c.getSumAmount())).append('\n');

    }

    private void appendTransaction(final Transaction t, final Writer writer)
    throws IOException
    {
        final NumberFormat ref11Fmt = new DecimalFormat("00000000000");
        final NumberFormat blzFmt = new DecimalFormat("00000000");
        final NumberFormat acctFmt = new DecimalFormat("0000000000");

        if(t.getReference() != null)
        {
            writer.append(ref11Fmt.format(t.getReference())).append(' ');
        }

        writer.append(t.getType().getShortDescription(Locale.getDefault()));

        if(t.getPrimaryBank() != null)
        {
            writer.append(' ').append(blzFmt.format(t.getPrimaryBank()));
        }

        writer.append("\n\n");

        writer.append(t.getExecutiveName()).append(' ').
            append(t.getTargetName());

        if(t.getDescriptions().length > 0)
        {
            writer.append(' ').append(t.getDescriptions()[0]);
        }

        writer.append('\n');

        writer.append(t.getExecutiveExt() == null ?
            "                           " : t.getExecutiveExt()).append(' ').
            append(t.getTargetExt() == null ?
                "                           " : t.getTargetExt());

        if(t.getDescriptions().length > 1)
        {
            writer.append(' ').append(t.getDescriptions()[1]);
        }

        writer.append('\n');

        writer.append(blzFmt.format(t.getExecutiveBank())).
            append("                    ").
            append("                   ").
            append(blzFmt.format(t.getTargetBank()));

        if(t.getDescriptions().length > 2)
        {
            writer.append(' ').append(t.getDescriptions()[2]);
        }

        writer.append('\n');

        writer.append(acctFmt.format(t.getExecutiveAccount())).
            append("                  ").
            append("                 ").
            append(acctFmt.format(t.getTargetAccount()));

        if(t.getDescriptions().length > 3)
        {
            writer.append(' ').append(t.getDescriptions()[3]);
        }

        writer.append('\n');

        for(int i = 4; i < t.getDescriptions().length; i++)
        {
            writer.append("                                        ").
                append("                ").
                append(t.getDescriptions()[i]).append('\n');

        }

        final BigDecimal divisor = new BigDecimal(Math.pow(10,
            t.getCurrency().getDefaultFractionDigits()));

        final BigDecimal amount = new BigDecimal(t.getAmount()).
            divide(divisor, t.getCurrency().getDefaultFractionDigits(),
            RoundingMode.HALF_UP);

        final NumberFormat amountFmt = NumberFormat.getCurrencyInstance();

        amountFmt.setMinimumFractionDigits(t.getCurrency().
            getDefaultFractionDigits());

        amountFmt.setMaximumFractionDigits(t.getCurrency().
            getDefaultFractionDigits());

        amountFmt.setCurrency(t.getCurrency());

        writer.append('\n').append(amountFmt.format(amount)).append('\n');
    }

    //-----------------------------------------------PhysicalFileTextFormatter--

}
