/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.editor.spi.messages;

import java.util.Locale;
import org.jdtaus.core.text.Message;

/**
 * Message stating that a number of logical files is incompatible with a
 * set of given file types.
 */
public final class FileCountConstraintMessage extends Message
{
    //--Message-----------------------------------------------------------------

    /**
     * Gets the arguments to use for formatting the message text.
     *
     * @param locale the locale for the argumetns or {@code null} for
     * {@code Locale.getDefault()}.
     *
     * @return the arguments to use for formatting the message text
     * ([{@code fileCount}, {@code typeCount}]).
     */
    public Object[] getFormatArguments(final Locale locale)
    {
        return new Object[] {
            new Integer(this.getFileCount()),
            new Integer(this.getTypeCount())
        };
    }

    /**
     * Gets the text of the message.
     *
     * @return the text of the message.
     */
    public String getText(final Locale locale)
    {
        return FileCountConstraintMessageBundle.getFileCountConstraintMessage(
            locale).format(this.getFormatArguments(locale));

    }

    //-----------------------------------------------------------------Message--
    //--FileCountConstraintMessage----------------------------------------------

    /**
     * Number of logical files.
     * @serial
     */
    private int fileCount;

    /**
     * Number of given file types.
     * @serial
     */
    private int typeCount;

    /**
     * Creates a new {@code FileCountConstraintMessage} taking a number
     * of files and a number of file types.
     *
     * @param fileCount number of logical files.
     * @param typeCount number of file types.
     */
    public FileCountConstraintMessage(final int fileCount,
        final int typeCount)
    {
        this.fileCount = fileCount;
        this.typeCount = typeCount;
    }

    /**
     * Gets the number of logical files.
     *
     * @return number of logical files.
     */
    public int getFileCount()
    {
        return this.fileCount;
    }

    /**
     * Gets the number of file types.
     *
     * @return number of file types.
     */
    public int getTypeCount()
    {
        return this.typeCount;
    }

    //----------------------------------------------FileCountConstraintMessage--
}
